@@ -465,6 +465,8 @@ void display_fsinfo(struct fsal_module *fsal)
 		 fsal->fs_info.fsal_trace);
 	LogDebug(COMPONENT_FSAL, "  fsal_grace = %d  ",
 		 fsal->fs_info.fsal_grace);
+	LogDebug(COMPONENT_FSAL, "  expire_time_parent = %d  ",
+		 fsal->fs_info.expire_time_parent);
 	LogDebug(COMPONENT_FSAL, "}");
 }
 
@@ -985,7 +987,7 @@ static bool posix_get_fsid(struct fsal_filesystem *fs)
 #endif
 
 	if (stat(fs->path, &mnt_stat) != 0) {
-		LogCrit(COMPONENT_FSAL,
+		LogEvent(COMPONENT_FSAL,
 			"stat of %s resulted in error %s(%d)",
 			fs->path, strerror(errno), errno);
 		return false;
@@ -1370,7 +1372,7 @@ int resolve_posix_filesystem(const char *path,
 	return retval;
 }
 
-static void release_posix_file_system(struct fsal_filesystem *fs)
+void release_posix_file_system(struct fsal_filesystem *fs)
 {
 	struct fsal_filesystem *child_fs;
 
@@ -2801,6 +2803,8 @@ fsal_status_t fsal_find_fd(struct fsal_fd **out_fd,
 		status = open_func(obj_hdl, openflags, state_fd);
 
 		if (status.major == ERR_FSAL_ACCESS &&
+		   (state->state_type == STATE_TYPE_LOCK ||
+		    state->state_type == STATE_TYPE_NLM_LOCK) &&
 		    state->state_data.lock.openstate != NULL) {
 			/* Got an EACCESS and openstate is available, try
 			 * again with it's openflags.
