@@ -117,7 +117,7 @@ static fsal_status_t check_open_permission(struct fsal_obj_handle *obj,
 	 * verifier later, so this allows a replay of
 	 * open("foo", O_RDWR | O_CREAT | O_EXCL, 0) to succeed).
 	 */
-	status = obj->obj_ops.test_access(obj, access_mask,
+	status = obj->obj_ops->test_access(obj, access_mask,
 					  NULL, NULL, exclusive_create);
 
 	if (!FSAL_IS_ERROR(status)) {
@@ -193,7 +193,7 @@ static fsal_status_t fsal_check_setattr_perms(struct fsal_obj_handle *obj,
 							op_ctx->fsal_export)
 			   & (ATTRS_CREDS | ATTR_MODE | ATTR_ACL));
 
-	status = obj->obj_ops.getattrs(obj, current);
+	status = obj->obj_ops->getattrs(obj, current);
 
 	if (FSAL_IS_ERROR(status))
 		return status;
@@ -325,7 +325,7 @@ static fsal_status_t fsal_check_setattr_perms(struct fsal_obj_handle *obj,
 	}
 
 	if (current->acl) {
-		status = obj->obj_ops.test_access(obj, access_check, NULL,
+		status = obj->obj_ops->test_access(obj, access_check, NULL,
 						  NULL, false);
 		note = " (checked ACL)";
 		goto out;
@@ -338,7 +338,7 @@ static fsal_status_t fsal_check_setattr_perms(struct fsal_obj_handle *obj,
 		goto out;
 	}
 
-	status = obj->obj_ops.test_access(obj, FSAL_W_OK, NULL, NULL, false);
+	status = obj->obj_ops->test_access(obj, FSAL_W_OK, NULL, NULL, false);
 
 	note = " (checked mode)";
 
@@ -390,7 +390,7 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 	if (FSAL_IS_ERROR(status))
 		return status;
 
-	status = in_obj->obj_ops.open2(in_obj,
+	status = in_obj->obj_ops->open2(in_obj,
 				       state,
 				       openflags,
 				       createmode,
@@ -432,7 +432,7 @@ fsal_status_t open2_by_name(struct fsal_obj_handle *in_obj,
 		 reason, fsal_err_txt(status));
 
 	if (state != NULL)
-		close_status = (*obj)->obj_ops.close2(*obj, state);
+		close_status = (*obj)->obj_ops->close2(*obj, state);
 	else
 		close_status = fsal_close(*obj);
 
@@ -554,7 +554,7 @@ fsal_status_t fsal_setattr(struct fsal_obj_handle *obj, bool bypass,
 		attr->mode &= ~S_ISGID;
 	}
 
-	status = obj->obj_ops.setattr2(obj, bypass, state, attr);
+	status = obj->obj_ops->setattr2(obj, bypass, state, attr);
 	if (FSAL_IS_ERROR(status)) {
 		if (status.major == ERR_FSAL_STALE) {
 			LogEvent(COMPONENT_FSAL,
@@ -585,7 +585,7 @@ fsal_status_t fsal_readlink(struct fsal_obj_handle *obj,
 		return fsalstat(ERR_FSAL_BADTYPE, 0);
 
 	/* Never refresh.  FSAL_MDCACHE will override for cached FSALs. */
-	return obj->obj_ops.readlink(obj, link_content, false);
+	return obj->obj_ops->readlink(obj, link_content, false);
 }
 
 /**
@@ -635,7 +635,7 @@ fsal_status_t fsal_link(struct fsal_obj_handle *obj,
 
 	/* Rather than performing a lookup first, just try to make the
 	   link and return the FSAL's error if it fails. */
-	status = obj->obj_ops.link(obj, dest_dir, name);
+	status = obj->obj_ops->link(obj, dest_dir, name);
 	return status;
 }
 
@@ -673,14 +673,14 @@ fsal_status_t fsal_lookup(struct fsal_obj_handle *parent,
 		return fsal_status;
 
 	if (strcmp(name, ".") == 0) {
-		parent->obj_ops.get_ref(parent);
+		parent->obj_ops->get_ref(parent);
 		*obj = parent;
 		return get_optional_attrs(*obj, attrs_out);
 	} else if (strcmp(name, "..") == 0)
 		return fsal_lookupp(parent, obj, attrs_out);
 
 
-	return parent->obj_ops.lookup(parent, name, obj, attrs_out);
+	return parent->obj_ops->lookup(parent, name, obj, attrs_out);
 }
 
 /**
@@ -720,18 +720,18 @@ fsal_status_t fsal_lookupp(struct fsal_obj_handle *obj,
 				/* Need to return the attributes of the
 				 * current object.
 				 */
-				return obj->obj_ops.getattrs(obj, attrs_out);
+				return obj->obj_ops->getattrs(obj, attrs_out);
 			} else {
 				/* Success */
 				return fsalstat(ERR_FSAL_NO_ERROR, 0);
 			}
 		} else {
 			/* Return entry from nfs_export_get_root_entry */
-			root_obj->obj_ops.put_ref(root_obj);
+			root_obj->obj_ops->put_ref(root_obj);
 		}
 	}
 
-	return obj->obj_ops.lookup(obj, "..", parent, attrs_out);
+	return obj->obj_ops->lookup(obj, "..", parent, attrs_out);
 }
 
 /**
@@ -832,12 +832,12 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 		break;
 
 	case DIRECTORY:
-		status = parent->obj_ops.mkdir(parent, name, attrs,
+		status = parent->obj_ops->mkdir(parent, name, attrs,
 					       obj, attrs_out);
 		break;
 
 	case SYMBOLIC_LINK:
-		status = parent->obj_ops.symlink(parent, name, link_content,
+		status = parent->obj_ops->symlink(parent, name, link_content,
 						 attrs, obj, attrs_out);
 		break;
 
@@ -845,7 +845,7 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 	case FIFO_FILE:
 	case BLOCK_FILE:
 	case CHARACTER_FILE:
-		status = parent->obj_ops.mknode(parent, name, type,
+		status = parent->obj_ops->mknode(parent, name, type,
 						attrs, obj, attrs_out);
 		break;
 
@@ -873,7 +873,7 @@ fsal_status_t fsal_create(struct fsal_obj_handle *parent,
 					     "create failed because it already exists");
 				if ((*obj)->type != type) {
 					/* Incompatible types, returns NULL */
-					(*obj)->obj_ops.put_ref((*obj));
+					(*obj)->obj_ops->put_ref((*obj));
 					*obj = NULL;
 					goto out;
 				}
@@ -923,7 +923,7 @@ bool fsal_create_verify(struct fsal_obj_handle *obj, uint32_t verf_hi,
 
 	fsal_prepare_attrs(&attrs, ATTR_ATIME | ATTR_MTIME);
 
-	obj->obj_ops.getattrs(obj, &attrs);
+	obj->obj_ops->getattrs(obj, &attrs);
 	if (FSAL_TEST_MASK(attrs.valid_mask, ATTR_ATIME)
 	    && FSAL_TEST_MASK(attrs.valid_mask, ATTR_MTIME)
 	    && attrs.atime.tv_sec == verf_hi
@@ -936,143 +936,6 @@ bool fsal_create_verify(struct fsal_obj_handle *obj, uint32_t verf_hi,
 	return verified;
 }
 
-/**
- * @brief New style reads
- *
- * @param[in]     obj          File to be read or written
- * @param[in]     bypass       If state doesn't indicate a share reservation,
- *                             bypass any deny read
- * @param[in]     state        state_t associated with the operation
- * @param[in]     offset       Absolute file position for I/O
- * @param[in]     io_size      Amount of data to be read or written
- * @param[out]    bytes_moved  The length of data successfuly read
- * @param[in,out] buffer       Where in memory to read data
- * @param[out]    eof          Whether a READ encountered the end of file
- * @param[in]     info         io_info for READ_PLUS
- *
- * @return FSAL status
- */
-
-fsal_status_t fsal_read2(struct fsal_obj_handle *obj,
-			 bool bypass,
-			 struct state_t *state,
-			 uint64_t offset,
-			 size_t io_size,
-			 size_t *bytes_moved,
-			 void *buffer,
-			 bool *eof,
-			 struct io_info *info)
-{
-	/* Error return from FSAL calls */
-	fsal_status_t status = { 0, 0 };
-
-	status = obj->obj_ops.read2(obj, bypass, state, offset, io_size, buffer,
-				    bytes_moved, eof, info);
-
-	/* Fixup FSAL_SHARE_DENIED status */
-	if (status.major == ERR_FSAL_SHARE_DENIED)
-		status = fsalstat(ERR_FSAL_LOCKED, 0);
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "FSAL READ operation returned %s, asked_size=%zu, effective_size=%zu",
-		     fsal_err_txt(status), io_size, *bytes_moved);
-
-	if (FSAL_IS_ERROR(status)) {
-		*bytes_moved = 0;
-		return status;
-	}
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "inode/direct: io_size=%zu, bytes_moved=%zu, offset=%"
-		     PRIu64, io_size, *bytes_moved, offset);
-
-	if (!*eof) {
-		/** @todo FSF: add a config option for this behavior?
-		 */
-		/*
-		 * NFS requires to set the EOF flag for all reads that
-		 * reach the EOF, i.e., even the ones returning data.
-		 * Most FSALs don't set the flag in this case. The only
-		 * client that cares about this is ESXi. Other clients
-		 * will just see a short read and continue reading and then
-		 * get the EOF flag as 0 bytes are returned.
-		 */
-		struct attrlist attrs;
-
-		fsal_prepare_attrs(&attrs, ATTR_SIZE);
-		if (FSAL_IS_SUCCESS(obj->obj_ops.getattrs(obj, &attrs)))
-			*eof = (offset + *bytes_moved) >= attrs.filesize;
-		fsal_release_attrs(&attrs);
-	}
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
-}
-
-/**
- * @brief New style writes
- *
- * @param[in]     obj          File to be read or written
- * @param[in]     bypass       If state doesn't indicate a share reservation,
- *                             bypass any non-mandatory deny write
- * @param[in]     state        state_t associated with the operation
- * @param[in]     offset       Absolute file position for I/O
- * @param[in]     io_size      Amount of data to be written
- * @param[out]    bytes_moved  The length of data successfuly written
- * @param[in,out] buffer       Where in memory to write data
- * @param[in]     sync         Whether the write is synchronous or not
- * @param[in]     info         io_info for WRITE_PLUS
- *
- * @return FSAL status
- */
-
-fsal_status_t fsal_write2(struct fsal_obj_handle *obj,
-			  bool bypass,
-			  struct state_t *state,
-			  uint64_t offset,
-			  size_t io_size,
-			  size_t *bytes_moved,
-			  void *buffer,
-			  bool *sync,
-			  struct io_info *info)
-{
-	/* Error return from FSAL calls */
-	fsal_status_t status = { 0, 0 };
-
-	if (op_ctx->export_perms->options & EXPORT_OPTION_COMMIT) {
-		/* Force sync if export requires it */
-		*sync = true;
-	}
-
-	status = obj->obj_ops.write2(obj,
-				     bypass,
-				     state,
-				     offset,
-				     io_size,
-				     buffer,
-				     bytes_moved,
-				     sync,
-				     info);
-
-	/* Fixup ERR_FSAL_SHARE_DENIED status */
-	if (status.major == ERR_FSAL_SHARE_DENIED)
-		status = fsalstat(ERR_FSAL_LOCKED, 0);
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "FSAL WRITE operation returned %s, asked_size=%zu, effective_size=%zu",
-		     fsal_err_txt(status), io_size, *bytes_moved);
-
-	if (FSAL_IS_ERROR(status)) {
-		*bytes_moved = 0;
-
-		return status;
-	}
-
-	LogFullDebug(COMPONENT_FSAL,
-		     "inode/direct: io_size=%zu, bytes_moved=%zu, offset=%"
-		     PRIu64, io_size, *bytes_moved, offset);
-
-	return fsalstat(ERR_FSAL_NO_ERROR, 0);
-}
-
 struct fsal_populate_cb_state {
 	struct fsal_obj_handle *directory;
 	fsal_status_t *status;
@@ -1162,7 +1025,7 @@ populate_dirent(const char *name,
 					.fs_supported_attrs(op_ctx->fsal_export)
 					| ATTR_RDATTR_ERR);
 
-		status = junction_obj->obj_ops.getattrs(junction_obj, &attrs2);
+		status = junction_obj->obj_ops->getattrs(junction_obj, &attrs2);
 
 		if (!FSAL_IS_ERROR(status)) {
 			/* Now call the callback again with that. */
@@ -1183,8 +1046,14 @@ populate_dirent(const char *name,
 		/* Release our refs */
 		op_ctx->fsal_export = saved_export;
 
-		junction_obj->obj_ops.put_ref(junction_obj);
+		junction_obj->obj_ops->put_ref(junction_obj);
 		put_gsh_export(junction_export);
+
+		/* state->cb (nfs4_readdir_callback) saved op_ctx
+		 * ctx_export and fsal_export. Restore them here
+		 */
+		(void)state->cb(&state->cb_parms, NULL, NULL,
+				 0, 0, CB_PROBLEM);
 	}
 
 	if (!state->cb_parms.in_result) {
@@ -1198,7 +1067,7 @@ populate_dirent(const char *name,
 out:
 
 	/* Put the ref on obj that readdir took */
-	obj->obj_ops.put_ref(obj);
+	obj->obj_ops->put_ref(obj);
 
 	return retval;
 }
@@ -1292,7 +1161,7 @@ fsal_status_t fsal_readdir(struct fsal_obj_handle *directory,
 	state.cb_nfound = nbfound;
 	state.attrmask = attrmask;
 
-	fsal_status = directory->obj_ops.readdir(directory, &cookie,
+	fsal_status = directory->obj_ops->readdir(directory, &cookie,
 						 (void *)&state,
 						 populate_dirent,
 						 attrmask,
@@ -1358,7 +1227,7 @@ fsal_remove(struct fsal_obj_handle *parent, const char *name)
 		goto out;
 #endif /* ENABLE_RFC_ACL */
 
-	status = parent->obj_ops.unlink(parent, to_remove_obj, name);
+	status = parent->obj_ops->unlink(parent, to_remove_obj, name);
 
 	if (FSAL_IS_ERROR(status)) {
 		LogFullDebug(COMPONENT_FSAL, "unlink %s failure %s",
@@ -1368,7 +1237,7 @@ fsal_remove(struct fsal_obj_handle *parent, const char *name)
 
 out:
 
-	to_remove_obj->obj_ops.put_ref(to_remove_obj);
+	to_remove_obj->obj_ops->put_ref(to_remove_obj);
 
 out_no_obj:
 
@@ -1400,9 +1269,10 @@ fsal_status_t fsal_rename(struct fsal_obj_handle *dir_src,
 		return fsalstat(ERR_FSAL_NOTDIR, 0);
 
 	/* Check for . and .. on oldname and newname. */
-	if (!strcmp(oldname, ".") || !strcmp(oldname, "..")
+	if (oldname[0] == '\0' || newname[0] == '\0'
+	    || !strcmp(oldname, ".") || !strcmp(oldname, "..")
 	    || !strcmp(newname, ".") || !strcmp(newname, "..")) {
-		return fsalstat(ERR_FSAL_BADNAME, 0);
+		return fsalstat(ERR_FSAL_INVAL, 0);
 	}
 
 	/* Check for object existence in source directory */
@@ -1430,7 +1300,7 @@ fsal_status_t fsal_rename(struct fsal_obj_handle *dir_src,
 
 	LogFullDebug(COMPONENT_FSAL, "about to call FSAL rename");
 
-	fsal_status = dir_src->obj_ops.rename(lookup_src, dir_src, oldname,
+	fsal_status = dir_src->obj_ops->rename(lookup_src, dir_src, oldname,
 					      dir_dest, newname);
 
 	LogFullDebug(COMPONENT_FSAL, "returned from FSAL rename");
@@ -1450,7 +1320,7 @@ out:
 		 * export since that would be the junction node, NOT the export
 		 * root node on the other side of the junction.
 		 */
-		lookup_src->obj_ops.put_ref(lookup_src);
+		lookup_src->obj_ops->put_ref(lookup_src);
 	}
 
 	return fsal_status;
@@ -1552,7 +1422,7 @@ fsal_status_t fsal_open2(struct fsal_obj_handle *in_obj,
 	 * might be an exclusive recreate replay and we want the FSAL to
 	 * check the verifier.
 	 */
-	status = in_obj->obj_ops.open2(in_obj,
+	status = in_obj->obj_ops->open2(in_obj,
 				       state,
 				       openflags,
 				       createmode,
@@ -1566,7 +1436,7 @@ fsal_status_t fsal_open2(struct fsal_obj_handle *in_obj,
 	if (!FSAL_IS_ERROR(status)) {
 		/* Get a reference to the entry. */
 		*obj = in_obj;
-		in_obj->obj_ops.get_ref(in_obj);
+		in_obj->obj_ops->get_ref(in_obj);
 	}
 
 	return status;
@@ -1603,7 +1473,7 @@ fsal_status_t fsal_reopen2(struct fsal_obj_handle *obj,
 
 	/* Re-open the entry in the FSAL.
 	 */
-	status = obj->obj_ops.reopen2(obj, state, openflags);
+	status = obj->obj_ops->reopen2(obj, state, openflags);
 
  out:
 
@@ -1653,7 +1523,7 @@ fsal_status_t fsal_statfs(struct fsal_obj_handle *obj,
 fsal_status_t fsal_verify2(struct fsal_obj_handle *obj,
 			   fsal_verifier_t verifier)
 {
-	if (!obj->obj_ops.check_verifier(obj, verifier)) {
+	if (!obj->obj_ops->check_verifier(obj, verifier)) {
 		/* Verifier check failed. */
 		return fsalstat(ERR_FSAL_EXIST, 0);
 	}
@@ -1684,7 +1554,7 @@ fsal_status_t get_optional_attrs(struct fsal_obj_handle *obj_hdl,
 	if (attrs_out == NULL)
 		return fsalstat(ERR_FSAL_NO_ERROR, 0);
 
-	status = obj_hdl->obj_ops.getattrs(obj_hdl, attrs_out);
+	status = obj_hdl->obj_ops->getattrs(obj_hdl, attrs_out);
 
 	if (FSAL_IS_ERROR(status)) {
 		if (attrs_out->request_mask & ATTR_RDATTR_ERR) {
