@@ -668,7 +668,7 @@ state_status_t state_test(struct fsal_obj_handle *obj,
 state_status_t state_lock(struct fsal_obj_handle *obj,
 			  state_owner_t *owner,
 			  state_t *state, state_blocking_t blocking,
-			  state_block_data_t *block_data,
+			  state_block_data_t **block_data,
 			  fsal_lock_param_t *lock,
 			  /* owner that holds conflicting lock */
 			  state_owner_t **holder,
@@ -941,7 +941,6 @@ state_status_t state_nlm_share(struct fsal_obj_handle *obj,
 			       int share_deny,
 			       state_owner_t *owner,
 			       state_t *state,
-			       bool reclaim,
 			       bool unshare);
 
 void state_share_wipe(struct state_hdl *ostate);
@@ -982,11 +981,13 @@ void blocked_lock_polling(struct fridgethr_context *ctx);
  ******************************************************************************/
 
 /* Grace period handling */
+extern int32_t reclaim_completes; /* atomic */
 void nfs_start_grace(nfs_grace_start_t *gsp);
 void nfs_end_grace(void);
 bool nfs_in_grace(void);
 void nfs_maybe_start_grace(void);
 bool nfs_grace_is_member(void);
+int nfs_recovery_get_nodeid(char **pnodeid);
 void nfs_try_lift_grace(void);
 void nfs_wait_for_grace_enforcement(void);
 void nfs_notify_grace_waiters(void);
@@ -994,7 +995,6 @@ void nfs_notify_grace_waiters(void);
 /* v4 Client stable-storage database management */
 void nfs4_add_clid(nfs_client_id_t *);
 void nfs4_rm_clid(nfs_client_id_t *);
-void nfs4_recovery_reclaim_complete(nfs_client_id_t *clientid);
 void nfs4_chk_clid(nfs_client_id_t *);
 
 /* Delegation revocation tracking */
@@ -1046,6 +1046,7 @@ struct nfs4_recovery_backend {
 	void (*set_enforcing)(void);
 	bool (*grace_enforcing)(void);
 	bool (*is_member)(void);
+	int (*get_nodeid)(char **pnodeid);
 };
 
 void fs_backend_init(struct nfs4_recovery_backend **);
